"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toSensorValue = exports.isSensorValueListDTO = exports.isSensorValueDTO = exports.isBasicSensorValueDTOConstrained = exports.isBasicSensorValueDTO = void 0;
const SensorValue_1 = require("./SensorValue");
function isAbstractSensorValueDTO(toCheck) {
    return toCheck.name !== undefined && toCheck.size !== undefined;
}
function isBasicSensorValueDTO(toCheck) {
    return isAbstractSensorValueDTO(toCheck) && toCheck.type !== undefined;
}
exports.isBasicSensorValueDTO = isBasicSensorValueDTO;
function isBasicSensorValueDTOConstrained(toCheck) {
    return toCheck.values === undefined;
}
exports.isBasicSensorValueDTOConstrained = isBasicSensorValueDTOConstrained;
function isSensorValueDTO(toCheck) {
    return isBasicSensorValueDTO(toCheck) || !isSensorValueListDTO(toCheck);
}
exports.isSensorValueDTO = isSensorValueDTO;
function isSensorValueListDTO(toCheck) {
    try {
        if (!isAbstractSensorValueDTO(toCheck))
            return false;
        const values = toCheck.values;
        if (values === undefined)
            return false;
        for (const val of values) {
            if (!isBasicSensorValueDTO(val) || !isSensorValueListDTO(val)) {
                return false;
            }
        }
    }
    catch (error) {
        return false;
    }
    return true;
}
exports.isSensorValueListDTO = isSensorValueListDTO;
function toSensorValue(dto) {
    if (isBasicSensorValueDTOConstrained(dto)) {
        return new SensorValue_1.NumberSensorValue(dto.name, SensorValue_1.getNumberParser(dto.type, dto.size));
    }
    else {
        const sensorValues = dto.values.map(val => toSensorValue(val));
        return new SensorValue_1.SensorValueList(sensorValues);
    }
}
exports.toSensorValue = toSensorValue;
