"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageRegistry = void 0;
const Message_1 = require("./Message");
const MessageDTO_1 = require("./MessageDTO");
const getMessageDefinitionMessage = (registry) => {
    const messageDefinitionMessage = {
        id: 255,
        parse: (toParse) => {
            return messageDefinitionMessage.value.parse(toParse);
        },
        value: {
            name: "message",
            parse: (message) => {
                const decoder = new TextDecoder("utf-8");
                const messageID = new Uint8Array(message)[0];
                const descriptionString = decoder.decode(message.slice(1));
                try {
                    const messageDTO = JSON.parse(descriptionString);
                    if (MessageDTO_1.isSensorValueDTO(messageDTO)) {
                        const messageSensorValue = MessageDTO_1.toSensorValue(messageDTO);
                        const newMessage = new Message_1.Message(messageSensorValue, messageID);
                        newMessage.value = newMessage.value.replaceBasicSensorValues(registry.basicSensorValues);
                        //push new basic snesor values dominantly
                        const newBasicSensorVals = messageSensorValue.getBasicSensorValues();
                        registry.basicSensorValues = registry.basicSensorValues.filter(val => newBasicSensorVals.findIndex(el => el.name === val.name) === -1);
                        registry.basicSensorValues.push(...newBasicSensorVals);
                        messageDefinitionMessage.value.message = newMessage;
                        // remove old message with same id if it exists
                        const oldIndex = registry.messages.findIndex(m => m.id == newMessage.id);
                        if (oldIndex != -1) {
                            console.log("attention, added same message id again!");
                            registry.messages.splice(oldIndex, 1);
                        }
                        registry.addMessage(newMessage);
                    }
                }
                catch (error) {
                    return false;
                }
                return true;
            },
            serialize: () => {
                // this tells the sender to resend all messag definitions
                return new Uint8Array([]);
            },
            size: maxMessageDefinitionSize,
            replaceBasicSensorValues: val => messageDefinitionMessage.value,
            message: null,
            getBasicSensorValues: () => []
        },
        registry: registry
    };
    return messageDefinitionMessage;
};
class MessageRegistry {
    constructor(messages = [], onMessage = (s) => console.log(s)) {
        this.currentMessage = null;
        this.buffer = new ArrayBuffer(0);
        this.bufferView = new Uint8Array(this.buffer);
        this.currentPosition = 0;
        this.previousByteZero = false;
        this.basicSensorValues = [];
        this.messages = [];
        messages.push(getMessageDefinitionMessage(this));
        this.streamMessage = new Message_1.StreamMessage(onMessage);
        messages.push(this.streamMessage);
        messages.forEach(m => this.addMessage(m));
    }
    readData(data) {
        for (let nextValue of new Uint8Array(data)) {
            if (this.currentPosition >= (this.currentMessage !== null ? this.currentMessage.value : 0)) {
                this.currentPosition = 0;
            }
            if (this.previousByteZero) {
                if (nextValue == 0) {
                    //zero occured in message
                    this.bufferView[this.currentPosition] = nextValue;
                    this.currentPosition++;
                }
                else if (nextValue == 1) {
                    // End of message reached
                    if (this.currentMessage) {
                        this.currentMessage.parse(this.buffer.slice(0, this.currentPosition));
                        this.currentMessage = null;
                    }
                }
                else {
                    //start of new message
                    const index = this.messages.findIndex(m => m.id == nextValue);
                    this.currentMessage = index == -1 ? null : this.messages[index];
                }
                this.previousByteZero = false;
            }
            else {
                if (nextValue == 0) {
                    this.previousByteZero = true;
                }
                else {
                    this.bufferView[this.currentPosition] = nextValue;
                    this.currentPosition++;
                }
            }
        }
    }
    addMessage(toAdd) {
        this.messages.push(toAdd);
        this.buffer = new ArrayBuffer(Math.max(...this.messages.map(m => m.value.size)));
        this.bufferView = new Uint8Array(this.buffer);
    }
    set onMessage(onMessage) {
        this.streamMessage._value.onMessage = onMessage;
    }
    /** get Arraybuffer which is ready to be sent to the microcontroller from message */
    encodeMessage(message) {
        const toEncode = new Uint8Array(message.value.serialize());
        const target = [0, message.id];
        for (const byte of toEncode) {
            target.push(byte);
            if (byte === 0)
                target.push(0);
        }
        target.push(0, 1);
        return new Uint8Array(target);
    }
}
exports.MessageRegistry = MessageRegistry;
const maxMessageDefinitionSize = 10000;
