export declare abstract class SensorValue {
    name: string;
    size: number;
    abstract parse(message: ArrayBuffer): boolean;
    abstract getBasicSensorValues(): NumberSensorValue[];
    replaceBasicSensorValues: (knownSet: SensorValue[]) => SensorValue;
    abstract serialize(): ArrayBuffer;
}
export declare type NumberType = "uint8" | "uint16" | "uint32" | "int8" | "int16" | "int32" | "float32" | "float64";
declare type NumberParser = {
    name: NumberType;
    size: number;
    parse: (raw: ArrayBuffer) => number;
    serialize: (number: number) => ArrayBuffer;
};
export declare const parsers: {
    [k: string]: NumberParser;
};
/**
 *
 * @param numberType for example int or float
 * @param size in bytes
 * @returns number parser
 */
export declare const getNumberParser: (numberType: string, size: number) => NumberParser;
export declare class NumberSensorValue extends SensorValue {
    getBasicSensorValues(): NumberSensorValue[];
    parser: NumberParser;
    value: number;
    constructor(name: string, parser: NumberParser);
    parse(message: ArrayBuffer): boolean;
    serialize(): ArrayBuffer;
}
export declare class SensorValueList extends SensorValue {
    getBasicSensorValues(): NumberSensorValue[];
    sensorValues: SensorValue[];
    constructor(sensorValues: SensorValue[]);
    parse(message: ArrayBuffer): boolean;
    serialize(): Uint8Array;
}
export {};
