"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SensorValueList = exports.NumberSensorValue = exports.getNumberParser = exports.parsers = exports.SensorValue = void 0;
class SensorValue {
    constructor() {
        this.name = "";
        this.size = 0;
        this.replaceBasicSensorValues = (knownSet) => this;
    }
}
exports.SensorValue = SensorValue;
exports.parsers = {
    uint8: {
        name: "uint8",
        size: 1,
        parse: (raw) => new Uint8Array(raw)[0],
        serialize: (number) => new Uint8Array([number])
    },
    uint16: {
        name: "uint16",
        size: 2,
        parse: (raw) => new Uint16Array(raw)[0],
        serialize: (number) => new Uint16Array([number])
    },
    uint32: {
        name: "uint32",
        size: 4,
        parse: (raw) => new Uint32Array(raw)[0],
        serialize: (number) => new Uint32Array([number])
    },
    int8: {
        name: "int8",
        size: 1,
        parse: (raw) => new Int8Array(raw)[0],
        serialize: (number) => new Int8Array([number])
    },
    int16: {
        name: "int16",
        size: 2,
        parse: (raw) => new Int16Array(raw)[0],
        serialize: (number) => new Int16Array([number])
    },
    int32: {
        name: "int32",
        size: 4,
        parse: (raw) => new Int32Array(raw)[0],
        serialize: (number) => new Int32Array([number])
    },
    float32: {
        name: "float32",
        size: 4,
        parse: (raw) => new Float32Array(raw)[0],
        serialize: (number) => new Float32Array([number])
    },
    float64: {
        name: "float64",
        size: 4,
        parse: (raw) => new Float64Array(raw)[0],
        serialize: (number) => new Float64Array([number])
    },
};
/**
 *
 * @param numberType for example int or float
 * @param size in bytes
 * @returns number parser
 */
const getNumberParser = (numberType, size) => {
    return exports.parsers[numberType + (8 * size)];
};
exports.getNumberParser = getNumberParser;
class NumberSensorValue extends SensorValue {
    constructor(name, parser) {
        super();
        this.value = 0;
        this.name = name;
        this.parser = parser;
        this.size = parser.size;
        this.replaceBasicSensorValues = (knownValues) => {
            const foundValue = knownValues.find(val => val.name == this.name);
            return foundValue == undefined ? this : foundValue;
        };
    }
    getBasicSensorValues() {
        return [this];
    }
    parse(message) {
        this.value = this.parser.parse(message);
        return true;
    }
    serialize() {
        return this.parser.serialize(this.value);
    }
}
exports.NumberSensorValue = NumberSensorValue;
class SensorValueList extends SensorValue {
    constructor(sensorValues) {
        super();
        this.sensorValues = sensorValues;
        this.sensorValues.forEach(val => this.size += val.size);
        this.replaceBasicSensorValues = (knownSet) => {
            for (const i in this.sensorValues) {
                this.sensorValues[i] = this.sensorValues[i].replaceBasicSensorValues(knownSet);
            }
            return this;
        };
    }
    getBasicSensorValues() {
        return this.sensorValues.filter(val => val instanceof NumberSensorValue);
    }
    parse(message) {
        let position = 0;
        for (let sensorValue of this.sensorValues) {
            sensorValue.parse(message.slice(position, position + sensorValue.size));
            position += sensorValue.size;
        }
        return position == this.size;
    }
    serialize() {
        const toReturn = new Uint8Array(this.size);
        let position = 0;
        for (let sens of this.sensorValues)
            [
                toReturn.set(new Uint8Array(sens.serialize()), 0)
            ];
        return toReturn;
    }
}
exports.SensorValueList = SensorValueList;
